package sun.security.jgss.krb5;

import java.security.Provider;
import java.io.UnsupportedEncodingException;
import java.util.Locale;
import java.net.UnknownHostException;
import java.net.InetAddress;
import javax.security.auth.kerberos.ServicePermission;
import sun.security.krb5.Realm;
import sun.security.krb5.KrbException;
import org.ietf.jgss.GSSException;
import org.ietf.jgss.GSSName;
import org.ietf.jgss.Oid;
import sun.security.krb5.PrincipalName;
import sun.security.jgss.spi.GSSNameSpi;

public class Krb5NameElement implements GSSNameSpi
{
    private PrincipalName krb5PrincipalName;
    private String gssNameStr;
    private Oid gssNameType;
    private static String CHAR_ENCODING;
    
    private Krb5NameElement(PrincipalName krb5PrincipalName, String gssNameStr, Oid gssNameType) {
        this.gssNameStr = null;
        this.gssNameType = null;
        this.krb5PrincipalName = krb5PrincipalName;
        this.gssNameStr = gssNameStr;
        this.gssNameType = gssNameType;
    }
    
    static Krb5NameElement getInstance(String s, Oid nt_GSS_KRB5_PRINCIPAL) throws GSSException {
        if (nt_GSS_KRB5_PRINCIPAL == null) {
            nt_GSS_KRB5_PRINCIPAL = Krb5MechFactory.NT_GSS_KRB5_PRINCIPAL;
        }
        else if (!nt_GSS_KRB5_PRINCIPAL.equals(GSSName.NT_USER_NAME) && !nt_GSS_KRB5_PRINCIPAL.equals(GSSName.NT_HOSTBASED_SERVICE) && !nt_GSS_KRB5_PRINCIPAL.equals(Krb5MechFactory.NT_GSS_KRB5_PRINCIPAL) && !nt_GSS_KRB5_PRINCIPAL.equals(GSSName.NT_EXPORT_NAME)) {
            throw new GSSException(4, -1, nt_GSS_KRB5_PRINCIPAL.toString() + " is an unsupported nametype");
        }
        Object o;
        try {
            if (nt_GSS_KRB5_PRINCIPAL.equals(GSSName.NT_EXPORT_NAME) || nt_GSS_KRB5_PRINCIPAL.equals(Krb5MechFactory.NT_GSS_KRB5_PRINCIPAL)) {
                o = new PrincipalName(s, 1);
            }
            else {
                String[] components = getComponents(s);
                if (nt_GSS_KRB5_PRINCIPAL.equals(GSSName.NT_USER_NAME)) {
                    o = new PrincipalName(s, 1);
                }
                else {
                    String s2 = null;
                    String s3 = components[0];
                    if (components.length >= 2) {
                        s2 = components[1];
                    }
                    o = new PrincipalName(getHostBasedInstance(s3, s2), 3);
                }
            }
        }
        catch (KrbException ex) {
            throw new GSSException(3, -1, ex.getMessage());
        }
        if (o.isRealmDeduced() && !Realm.AUTODEDUCEREALM) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                try {
                    securityManager.checkPermission(new ServicePermission("@" + o.getRealmAsString(), "-"));
                }
                catch (SecurityException ex2) {
                    throw new GSSException(11);
                }
            }
        }
        return new Krb5NameElement(o, s, nt_GSS_KRB5_PRINCIPAL);
    }
    
    public static Krb5NameElement getInstance(PrincipalName principalName) {
        return new Krb5NameElement(principalName, principalName.getName(), Krb5MechFactory.NT_GSS_KRB5_PRINCIPAL);
    }
    
    private static String[] getComponents(String s) throws GSSException {
        int lastIndex = s.lastIndexOf(64, s.length());
        if (lastIndex > 0 && s.charAt(lastIndex - 1) == '\\' && (lastIndex - 2 < 0 || s.charAt(lastIndex - 2) != '\\')) {
            lastIndex = -1;
        }
        if (lastIndex > 0) {
            return new String[] { s.substring(0, lastIndex), s.substring(lastIndex + 1) };
        }
        return new String[] { s };
    }
    
    private static String getHostBasedInstance(String str, String str2) throws GSSException {
        StringBuffer sb = new StringBuffer(str);
        try {
            if (str2 == null) {
                str2 = InetAddress.getLocalHost().getHostName();
            }
        }
        catch (UnknownHostException ex) {}
        str2 = str2.toLowerCase(Locale.ENGLISH);
        return sb.append('/').append(str2).toString();
    }
    
    public final PrincipalName getKrb5PrincipalName() {
        return this.krb5PrincipalName;
    }
    
    @Override
    public boolean equals(GSSNameSpi gssNameSpi) throws GSSException {
        return gssNameSpi == this || (gssNameSpi instanceof Krb5NameElement && this.krb5PrincipalName.getName().equals(((Krb5NameElement)gssNameSpi).krb5PrincipalName.getName()));
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        try {
            if ((o instanceof Krb5NameElement)) {
                return this.equals((Krb5NameElement)o);
            }
        }
        catch (GSSException ex) {}
        return false;
    }
    
    @Override
    public int hashCode() {
        return 629 + this.krb5PrincipalName.getName().hashCode();
    }
    
    @Override
    public byte[] export() throws GSSException {
        byte[] bytes = null;
        try {
            bytes = this.krb5PrincipalName.getName().getBytes(Krb5NameElement.CHAR_ENCODING);
        }
        catch (UnsupportedEncodingException ex) {}
        return bytes;
    }
    
    @Override
    public Oid getMechanism() {
        return Krb5MechFactory.GSS_KRB5_MECH_OID;
    }
    
    @Override
    public String toString() {
        return this.gssNameStr;
    }
    
    public Oid getGSSNameType() {
        return this.gssNameType;
    }
    
    @Override
    public Oid getStringNameType() {
        return this.gssNameType;
    }
    
    @Override
    public boolean isAnonymousName() {
        return this.gssNameType.equals(GSSName.NT_ANONYMOUS);
    }
    
    @Override
    public Provider getProvider() {
        return Krb5MechFactory.PROVIDER;
    }
    
    static {
        Krb5NameElement.CHAR_ENCODING = "UTF-8";
    }
}
